import os

def copy_ranges(source_path, dest_path, operations):
    """
    Copies multiple ranges of data from the source file to the destination file.

    :param source_path: Path to the source file.
    :param dest_path: Path to the destination file.
    :param operations: A list of tuples containing (copy_start, paste_start, length).
    """
    try:
        with open(source_path, 'rb') as source_file, open(dest_path, 'r+b') as dest_file:
            for copy_start, paste_start, length in operations:
                # Read the specified range from the source file
                source_file.seek(copy_start)
                data = source_file.read(length)
                
                # Write the data to the specified range in the destination file
                dest_file.seek(paste_start)
                dest_file.write(data)
        
    except FileNotFoundError:
        print("Error: File not found. Please check the path.")
    except Exception as e:
        print(f"An error occurred: {e}")

def main():
    print("This ports palettes from the Dreamcast Version to the Steam version")

    # Ask for the directory containing the DAT.BIN files once
    directory = input("Please enter the directory containing the DAT.BIN files: ").strip()

    # Define source and destination operations for each file
    files_operations = {
    "PL00_DAT.BIN": [
        (0x0006E440, 0x082CC60, 0x00000720)  # Ryu
    ],
    "PL01_DAT.BIN": [
        (0x000c7180, 0x09049A0, 0x00000D40)  # Zangief
    ],
    "PL02_DAT.BIN": [
        (0x00082200, 0x0997A20, 0x00000720)  # Guile
    ],
    "PL03_DAT.BIN": [
        (0x000b4b00, 0x0A5D320, 0x00001780)  # Morrigan
    ],
    "PL04_DAT.BIN": [
        (0x00108b40, 0x0B77B60, 0x00000A00)  # Anakaris
    ],
    "PL05_DAT.BIN": [
        (0x000c7600, 0x0C4FE20, 0x00000720)  # Strider Hiryu
    ],
    "PL06_DAT.BIN": [
        (0x000d6760, 0x0D36F80, 0x00001280)  # Cyclops
    ],
    "PL07_DAT.BIN": [
        (0x000ea500, 0x0E32D20, 0x00000840)  # Wolverine
    ],
    "PL08_DAT.BIN": [
        (0x000f1120, 0x0F34940, 0x00000740)  # Psylocke
    ],
    "PL09_DAT.BIN": [
        (0x00da9c0, 0x10201E0, 0x00000F40)  # Iceman
    ],
    "PL0A_DAT.BIN": [
        (0x00d5c60, 0x1107480, 0x00000A40)  # Rogue
    ],
    "PL0B_DAT.BIN": [
        (0x00dd880, 0x11F58A0, 0x00000960)  # Captain America
    ],
    "PL0C_DAT.BIN": [
        (0x00c9ca0, 0x12D44C0, 0x00001300)  # Spider-Man
    ],
    "PL0D_DAT.BIN": [
        (0x00109760, 0x13EEF80, 0x00000740)  # Hulk
    ],
    "PL0E_DAT.BIN": [
        (0x00113600, 0x1512E20, 0x00000720)  # Venom
    ],
    "PL0F_DAT.BIN": [
        (0x00102220, 0x1625A40, 0x00001C00)  # Dr. Doom
    ],
    "PL10_DAT.BIN": [
        (0x00105fa0, 0x173D7C0, 0x000008A0)  # Tron Bonne
    ],
    "PL11_DAT.BIN": [
        (0x000cb320, 0x1819B40, 0x00000760)  # Jill
    ],
    "PL12_DAT.BIN": [
        (0x000ecea0, 0x19176C0, 0x000008E0)  # Hayato
    ],
    "PL13_DAT.BIN": [
        (0x000f8ce0, 0x1A20D00, 0x00000960)  # Ruby Heart
    ],
    "PL14_DAT.BIN": [
        (0x001090a0, 0x1B3A8C0, 0x00000B00)  # SonSon
    ],
    "PL15_DAT.BIN": [
        (0x001085c0, 0x1C53DE0, 0x00000760)  # Amingo
    ],
    "PL16_DAT.BIN": [
        (0x00e4600, 0x1D48E20, 0x00000720)  # Marrow
    ],
    "PL17_DAT.BIN": [
        (0x00fba00, 0x1E55220, 0x00000720)  # Cable
    ],
    "PL18_DAT.BIN": [
        (0x00db7e0, 0x1F41800, 0x00000720)  # Abyss (Form 1)
    ],
    "PL19_DAT.BIN": [
        (0x0007cd60, 0x1FCED80, 0x00000720)  # Abyss (Form 2)
    ],
    "PL1A_DAT.BIN": [
        (0x00c9580, 0x20A8DA0, 0x00000720)  # Abyss (Form 3)
    ],
    "PL1B_DAT.BIN": [
        (0x006fd00, 0x2129520, 0x00000720)  # Chun-Li
    ],
    "PL1C_DAT.BIN": [
        (0x00082900, 0x21BC920, 0x00004840)  # Megaman
    ],
    "PL1D_DAT.BIN": [
        (0x0005f5c0, 0x2230DE0, 0x00004840)  # Roll
    ],
    "PL1E_DAT.BIN": [
        (0x00074d80, 0x22BA5A0, 0x00000720)  # Akuma
    ],
    "PL1F_DAT.BIN": [
        (0x00109a80, 0x23D4AA0, 0x00000960)  # B.B. Hood
    ],
    "PL20_DAT.BIN": [
        (0x001168e0, 0x24FC100, 0x00000720)  # Felicia
    ],
    "PL21_DAT.BIN": [
        (0x0005c980, 0x25691A0, 0x00000720)  # Charlie
    ],
    "PL22_DAT.BIN": [
        (0x00b6360, 0x2630380, 0x00000a40)  # Sakura
    ],
    "PL23_DAT.BIN": [
        (0x003db40, 0x267EB60, 0x00000720)  # Dan
    ],
    "PL24_DAT.BIN": [
        (0x008f3c0, 0x271EBE0, 0x00000dc0)  # Cammy
    ],
    "PL25_DAT.BIN": [
        (0x00a5f00, 0x27D5F20, 0x00000ac0)  # Dhalsim
    ],
    "PL26_DAT.BIN": [
        (0x0074080, 0x285B0A0, 0x00000880)  # M.Bison
    ],
    "PL27_DAT.BIN": [
        (0x0075ac0, 0x28E1AE0, 0x00000720)  # Ken
    ],
    "PL28_DAT.BIN": [
        (0x00d8f20, 0x29CB740, 0x00000ac0)  # Gambit
    ],
    "PL29_DAT.BIN": [
        (0x011c040, 0x2AF8860, 0x00000e80)  # Juggernaut
    ],
    "PL2A_DAT.BIN": [
        (0x00fd8c0, 0x2C070E0, 0x00000b40)  # Storm
    ],
    "PL2B_DAT.BIN": [
        (0x00f09c0, 0x2D089E0, 0x00000720)  # Sabretooth
    ],
    "PL2C_DAT.BIN": [
        (0x00106760, 0x2E1FF80, 0x00000740)  # Magneto
    ],
    "PL2D_DAT.BIN": [
        (0x00d7d40, 0x2F08560, 0x00002c00)  # Shuma-Gorath
    ],
    "PL2E_DAT.BIN": [
        (0x00ed980, 0x30091A0, 0x00000720)  # War Machine
    ],
    "PL2F_DAT.BIN": [
        (0x010aea0, 0x3124EC0, 0x00000d00)  # Silver Samurai
    ],
    "PL30_DAT.BIN": [
        (0x00ebb40, 0x3223B60, 0x00000a20)  # Omega Red
    ],
    "PL31_DAT.BIN": [
        (0x00104c60, 0x3339480, 0x00002040)  # Spiral
    ],
    "PL32_DAT.BIN": [
        (0x00119aa0, 0x34652C0, 0x00001f00)  # Colossus
    ],
    "PL33_DAT.BIN": [
        (0x00f0b80, 0x35683A0, 0x00000760)  # Iron Man
    ],
    "PL34_DAT.BIN": [
        (0x00115820, 0x368E840, 0x00000940)  # Sentinel
    ],
    "PL35_DAT.BIN": [
        (0x001166c0, 0x37B5EE0, 0x00000e00)  # Blackheart
    ],
    "PL36_DAT.BIN": [
        (0x00ddc60, 0x38A4C80, 0x00000a20)  # Thanos
    ],
    "PL37_DAT.BIN": [
        (0x00d3c40, 0x3989460, 0x000016e0)  # Jin
    ],
    "PL38_DAT.BIN": [
        (0x00de380, 0x3A793A0, 0x00000920)  # Captain Commando
    ],
    "PL39_DAT.BIN": [
        (0x00eeae0, 0x3B78B00, 0x00000840)  # Wolverine (Bone Claw)
    ],
    "PL3A_DAT.BIN": [
        (0x00055300, 0x3BDEB20, 0x00000b80)  # Servbot
    ]
}

    # Ask for the destination file path once
    dest_file = input("Please enter the full path to the destination .bin file for all operations: ").strip()
    
    # Check if the destination file exists
    if not os.path.isfile(dest_file):
        print(f"Error: {dest_file} does not exist. Exiting.")
        return

    # Iterate over each file and its corresponding operations
    for file_name, operations in files_operations.items():
        # Construct the full path to the current DAT.BIN file
        source_file = os.path.join(directory, file_name)
        
        # Check if the source file exists
        if not os.path.isfile(source_file):
            print(f"Error: {source_file} does not exist. Skipping this file.")
            continue

        # Copy the specified ranges from the source file to the destination file
        copy_ranges(source_file, dest_file, operations)

    print("The Palettes have been copied.")

if __name__ == "__main__":
    main()
